/* TODO: Recomment, add recount button */

#define DEBOUNCE_K 1
/*The debounce coefficient for the clock input single*/
#define MFIRSTREPEAT 60
#define MSECONDREPEAT 20
#define WAIT 20
#define True 1
#define False 0

#define ReadPin(port,pin) ((port) & _BV(pin))
#define SetHigh(port,pin) ((port) |= _BV(pin))
#define SetBitsHigh(port,bits) ((port) |= bits)
#define SetLow(port,pin) ((port) &= ~_BV(pin)) 
#define SetBitsLow(port,bits) ((port) &= ~(bits)) 


int main(void);

#include <avr/interrupt.h>    /* Defines pins, ports, etc to make programs easier to read */
#define F_CPU 1000000UL/*for the timer*/
#include <util/delay.h>/*_delay_ms stuff*/

#define MINUTEROLLOVER 40;
#define HOURROLLOVER 7600;
#define HOURINCREMENT 100;
#define CLOCKIN PD0
#define DEBUG PD1
#define BUTTON_MIN PD2
#define BUTTON_HOU PD3
#define OUT PD5

int main(){
	DDRD = _BV(OUT) + _BV(DEBUG); //OUT  & debug as outputs
	PIND = _BV(BUTTON_MIN) + _BV(BUTTON_HOU);//Pullup enabled on input buttons
	DDRB = ~0; //All of port B output
	short int MS = 0;	/* Milliseconds/trigger pulses */
	short int Seconds = 0;	/* Next 3 vars hold the time */
	short int Minutes = 0;
	short int Hours = 12;
	short int Pulse = 0;
	short int  HouPressed = 0;
	short int  MinPressed = 0;
	int CountUp = 1200;

	/*Set up the ports*/
	while(1){
		PORTB = ~Seconds;//seconds as true binary.  
		/*
		 * This snippet is 1/2 of the code to handle the actual pulse 
		 * for the counter circuit.  
		 * As long as CountUp is one or more, a pulse is generated to 
		 * increment the counter,  and CountUp is reduced by one.  
		 * This allows easy stacking of times (just add the number of
		 * pulses needed to CountUp) and the placement limits the
		 * speed of the output pulses to the speed of the 60hz signal
		 * that drives the clock.  This prevents extra pulses from 
		 * signal bounce that happens at higher frequency, as well
		 * as makes the output more interesting as you can see the 
		 * numbers tic away better during rollover.  It takes 2/3 of
		 * a second for the clock to go up to the next hour, and 
		 * several seconds for the clock to go back to midnight.
		 *
		 */
		if(CountUp){
			SetHigh(PORTD, OUT);
			CountUp--;
		}
		/*
		 * This segment deals with user inputs.  The inputs need to be 
		 * debounced, and this is accomplished by setting a flag
		 * if the button is read as pressed, and continuing on with the
		 * code.  Once the cycle has been completed (60ms), if the 
		 * button is still pressed, it then preforms the action.  If 
		 * the button is not pressed, the flag goes false.  
		 *
		 * The user inputs consist of  a minute button and an hour
		 * button.  The minute button causes the counter to increment
		 * at a significantly higher speed.  Pressing it increments
		 * time by one, holding it down for MFIRSTREPEAT tics
		 * increments again, and it will increment every MSECONDREPEAT 
		 * tics thereafter.  Hour only increments once per pressing.  
		 *
		 * In order to increment the time, minutes causes the 
		 * millisecond and second count to go to 60, so that the clock
		 * will think that it is the end of the minute and increment
		 * and roll over accordingly.  This is done to simplify the
		 * code, as now all of the rollover function is done in one
		 * part of the code.  It is not possible to do this with 
		 * the hour code, and as such there is code to handle the 
		 * rollover.  
		 * 
		 */

		if(!ReadPin(PIND, BUTTON_HOU)){//Hour button
			if(HouPressed == 1){
				HouPressed = 2;//So it will only run once
				Hours++;//up the hours & 
				if(Hours >= 24){//Rollback to zero
					CountUp += HOURROLLOVER + HOURINCREMENT;
					Hours = 0;
				}
				else{
					CountUp += HOURINCREMENT;
				}
			}
			if(HouPressed == 0){
				HouPressed = 1;
			}
		}
		else{
			HouPressed = 0;
		}
		if(!ReadPin(PIND, BUTTON_MIN)){//minute button
			if(MinPressed == 1){
				MS = 59;
				Seconds = 59;
			}
			if(MinPressed == MFIRSTREPEAT){//Held down for 
							//FIRSTREPEAT tics
				MS = 59;
				Seconds = 59;
			}

			if(MinPressed >= MFIRSTREPEAT &&    //It's been more 
				MinPressed % MSECONDREPEAT){//than FIRSTREPEAT
							    //and every SECOND
							    //REPEAT.
				MS = 59;
				Seconds = 59;
			}
			MinPressed += 1;//Update tics it's been held for.
		}

		else{
			MinPressed = 0;
		}
		/*
		 * Pulse detection routine.  Also handles the timing for the
		 * output pulse, This limits the speed of the clock so that it 
		 * is more visually interesting when rolling over and there are
		 * fewer bounce problems due to high frequency signals.  
		 * It works by waiting for the pulse to go low (with a mandatory
		 * debounce time of DEBOUNCE_K milliseconds) and letting it go
		 * up with the same debounce, and then proceeding with the code
		 * Debounce is handled by checking to see if the clock is 
		 * High (or low, if that's the part of the signal that is being
		 * read), waiting for DEBOUNCE_K, and then checking again.  A
		 * flag (Pulse) is used to see if the input was in the proper 
		 * state previously.
		 *
		 */
		while(1){
			//Wait until it goes low
			if(!ReadPin(PIND, CLOCKIN)){
				if(Pulse){
					break;
				}
				Pulse = 1;
			}
			_delay_ms(DEBOUNCE_K);
		}
		SetHigh(PIND, DEBUG);
		Pulse = 0;
		/* 
		 * This is the other half of the counter control, bringing
		 * the pulse low.  The placement means that the minimum pulse
		 * width will be DEBOUNCE_K milliseconds, and the maximum time
		 * will be 16 1/3 milliseconds, or half the wave of a 60hz 
		 * signal.
		 *
		 */
		SetLow(PORTD, OUT);
		while(1){
			//Wait until it goes high
			if(ReadPin(PIND, CLOCKIN)){
				if(Pulse){
					break;
				}
				Pulse = 1;
			}
			_delay_ms(DEBOUNCE_K);
		}
		Pulse = 0;

		MS++;//It's been a tic.
		/*
		 * This segment handles the timekeeping and rollover functions. 
		 * Every 60 tics, a second has passed.  After 60 seconds, the
		 * counter is incremented by one.  After 60 minutes, the counter
		 * is incremented by MINUTEROLLOVER, which is a constant chosen
		 * to cause the clock to go up to the next even century value.
		 * After 24 hours, the counter is incremented by HOURROLLOVER 
		 * to cause the counter to rap back around to zero.  
		 * 
		 */
		if(MS >= 60) {
			MS = 0;
			Seconds++;
			if(Seconds >= 60){//Advance a minute
				CountUp += 1;
				Seconds = 0;
				Minutes++;
				if(Minutes == 60){//Advance an hour
					CountUp += MINUTEROLLOVER;
					Minutes = 0;
					MinPressed = 2;
					Hours++;
					if(Hours == 24){//Rollback to zero
						CountUp += HOURROLLOVER;
						Hours = 0;
					}
				}
			}
		}
	}
	return(0);
}

